<?php declare(strict_types=1);

namespace Plugin\jtl_samplepayment\paymentmethod;

use JTL\Checkout\Bestellung;
use JTL\Plugin\Data\PaymentMethod;
use JTL\Plugin\Helper as PluginHelper;
use JTL\Plugin\Payment\Method;
use JTL\Plugin\PluginInterface;
use JTL\Shop;

/**
 * Class SamplePayment
 * @package Plugin\jtl_samplepayment\paymentmethod
 */
class SamplePayment extends Method
{
    /** @var PluginInterface */
    private PluginInterface $plugin;

    /** @var PaymentMethod|null */
    private ?PaymentMethod $method;

    /** @var bool */
    private bool $payAgain;

    /**
     * @inheritDoc
     */
    public function init(int $nAgainCheckout = 0): self
    {
        parent::init($nAgainCheckout);

        $pluginID       = PluginHelper::getIDByModuleID($this->moduleID);
        $this->plugin   = PluginHelper::getLoaderByPluginID($pluginID)->init($pluginID);
        $this->method   = $this->plugin->getPaymentMethods()->getMethodByID($this->moduleID);
        $this->payAgain = $nAgainCheckout > 0;

        return $this;
    }

    /**
     * @return PaymentMethod
     */
    public function getMethod(): PaymentMethod
    {
        return $this->method;
    }

    /**
     * @inheritDoc
     */
    public function getSetting(string $key): mixed
    {
        $setting = parent::getSetting($key);

        if ($setting === null) {
            $setting = $this->plugin->getConfig()->getValue($this->getMethod()->getModuleID() . '_' . $key);
        }

        return $setting;
    }

    /**
     * @inheritDoc
     */
    public function isValidIntern(array $args_arr = []): bool
    {
        if ($this->method === null) {
            return false;
        }

        $name     = $this->getSetting('payment_information_name') ?? '';
        $bankName = $this->getSetting('payment_information_bankname') ?? '';
        $IBAN     = $this->getSetting('payment_information_iban') ?? '';

        return parent::isValidIntern($args_arr) && $name !== '' && $bankName !== '' && $IBAN !== '';
    }

    /**
     * @inheritDoc
     */
    public function finalizeOrder(Bestellung $order, string $hash, array $args): bool
    {
        parent::finalizeOrder($order, $hash, $args);

        $payed = isset($args['payed']);
        $this->doLog('SamplePayment::finalizeOrder ' . ($payed ? 'payed' : 'not payed'));

        return $payed;
    }

    /**
     * @inheritDoc
     */
    public function redirectOnPaymentSuccess(): bool
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function redirectOnCancel(): bool
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function handleNotification(Bestellung $order, string $hash, array $args): void
    {
        parent::handleNotification($order, $hash, $args);

        if (isset($args['payed'])) {
            $this->addIncomingPayment($order, (object)[
                'fBetrag'           => $order->fGesamtsumme,
                'fZahlungsgebuehr'  => 0,
            ]);
            $this->setOrderStatusToPaid($order);
        }
    }

    /**
     * @inheritDoc
     */
    public function canPayAgain(): bool
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function preparePaymentProcess(Bestellung $order): void
    {
        parent::preparePaymentProcess($order);

        $smarty       = Shop::Smarty();
        $localization = $this->plugin->getLocalization();

        if ($this->payAgain) {
            $paymentHash = $this->getOrderHash($order);
            if ($paymentHash === null) {
                $this->getDB()->insert('tbestellid', (object)[
                    'kBestellung' => $order->kBestellung,
                    'cId'         => \uniqid('', true)
                ]);
                $paymentHash = $this->generateHash($order);
            }
        } else {
            $paymentHash = $this->generateHash($order);
        }

        Shop::Smarty()
            ->assign('samplePaymentSuccessURL', $this->getNotificationURL($paymentHash) . '&payed')
            ->assign('samplePaymentDescription', $localization->getTranslation('head_info_about_your_payment'))
            ->assign('samplePayment_LabelPayee', $localization->getTranslation('info_payee'))
            ->assign('samplePayment_ConfigPayee', $this->getSetting('payment_information_name'))
            ->assign('samplePayment_LabelBankName', $localization->getTranslation('info_bank'))
            ->assign('samplePayment_ConfigBankName', $this->getSetting('payment_information_bankname'))
            ->assign('samplePayment_LabelIBAN', $localization->getTranslation('info_iban'))
            ->assign('samplePayment_ConfigIBAN', $this->getSetting('payment_information_iban'));

        if ($this->duringCheckout) {
            $smarty->assign('samplePaymentCancelURL', $this->getNotificationURL($paymentHash));
        } else {
            $smarty->assign('samplePaymentCancelURL', $this->getReturnURL($order));
        }
    }
}
